%% twogoodoptonecnsmis.m
%% adapted from twogoodoptonecnsmaxiter.m
%%
%% Gabriel Carroll
%% Feb 16, 2015
%%
%% Consider selling two goods to a buyer with additive valuations,
%% where we know the marginal distribution of values for each good
%% but don't know the joint distribution.  Want a mechanism that
%% maximizes min revenue over all joint distributions satisfying a single
%% linear constraint (in addition to the known marginals).  When are we 
%% still pinned down to the revenue we would get from separate sales?
%% This is intended as a step toward solving the "negative correlation"
%% question (does negative correlation imply we can definitely do better
%% than separate sales).
%%
%% Note my indexing of positions in the LP is stylistically poor - I've
%% used the same functions as without the extra one constraint (for the
%% most part) since I know there'll be only one extra variable (the
%% constraint multiplier) which I can tack on the end; ideally would use
%% a totally separate set of functions.
%%
%% Adapted to allow specification of max # iterations of the LP solver.
%% Differs from twogoodoptonecnsmaxiter.m in that we now use the simplex
%% LP solver instead of the default interior-point method.  Note that this
%% means wcdist is meaningless.
%%
%% Inputs:
%%    valuesa - 1 x k_1 vector of values for first good
%%    probsa - 1 x k_1 vector of corresponding probabilities
%%    valuesb - 1 x k_2 vector of values for second good
%%    probsb - 1 x k_2 vector of corresponding probabilities
%%    extracns - k_1 x k_2 matrix of coefficients for extra constraint
%%    extraconst - right-hand side of constraint:
%%                 constraint takes form
%%                     sum_{i,j} extracns(i,j)*prob(i,j) >= extraconst
%%    mi - max # iterations of solver
%%
%% Outputs;
%%    rev - optimum robust revenue
%%    mech - structure with fields: allocamat, allocbmat, paymat
%%           giving the optimal mechanism as k_1 x k_2 matrices
%%    wcdist - (this is a placeholder)
%%    seprev - revenue achievable by selling the goods separately

function [rev,mech,wcdist,seprev] = twogoodoptonecnsmis(valuesa,probsa,valuesb,probsb,extracns,extraconst,mi)

ka = size(valuesa,2);
kb = size(valuesb,2);
vsize = lpvarsizeonecns(ka,kb);
A = zeros(0,vsize);
b = zeros(0,1);

%% IC constraints

for va = 1:ka
    for vb = 1:kb
        for vap = 1:ka
            for vbp = 1:kb
                if (vap ~= va) || (vbp ~= vb)
                newcns = zeros(1,vsize);
                newcns(allocapos(ka,kb,va,vb)) = valuesa(1,va);
                newcns(allocbpos(ka,kb,va,vb)) = valuesb(1,vb);
                newcns(paypos(ka,kb,va,vb)) = -1;
                newcns(allocapos(ka,kb,vap,vbp)) = -valuesa(1,va);
                newcns(allocbpos(ka,kb,vap,vbp)) = -valuesb(1,vb);
                newcns(paypos(ka,kb,vap,vbp)) = 1;
                newcns = -newcns;
                A = [A;newcns];
                b = [b;0];
                end;
            end;
        end;
    end;
end;

%% IR constraints

for va = 1:ka
    for vb = 1:kb
        newcns = zeros(1,vsize);
        newcns(allocapos(ka,kb,va,vb)) = valuesa(1,va);
        newcns(allocbpos(ka,kb,va,vb)) = valuesb(1,vb);
        newcns(paypos(ka,kb,va,vb)) = -1;
        newcns = -newcns;
        A = [A;newcns];
        b = [b;0];
    end;
end;

%% dual constraints in worst-case-probability problem
%% keep track of where they lie in constraint list

dualcnspos = zeros(ka,kb);

for va = 1:ka
    for vb = 1:kb
        newcns = zeros(1,vsize);
        newcns(probadualpos(ka,kb,va)) = -1;
        newcns(probbdualpos(ka,kb,vb)) = -1;
        newcns(extracnsdualpos(ka,kb)) = -extracns(va,vb);
        newcns(paypos(ka,kb,va,vb)) = 1;
        newcns = -newcns;
        A = [A;newcns];
        b = [b;0];
        dualcnspos(va,vb) = size(A,1);
    end;
end;

%% objective is sum of duals in probability problem, 
%% weighted by marginal probability of each value
%% plus dual on extra constraint times its right-hand side 

obj = zeros(1,vsize);
for va = 1:ka
    obj(probadualpos(ka,kb,va)) = probsa(1,va);
end;
for vb = 1:kb
    obj(probbdualpos(ka,kb,vb)) = probsb(1,vb);
end;
obj(extracnsdualpos(ka,kb)) = extraconst;
obj = -obj;

%% lower and upper bounds

lb = zeros(1,vsize);
ub = ones(1,vsize);
for va = 1:ka
    for vb = 1:kb
        lb(paypos(ka,kb,va,vb)) = -Inf;
        ub(paypos(ka,kb,va,vb)) = Inf;
    end;
end;
for va = 1:ka
    lb(probadualpos(ka,kb,va)) = -Inf;
    ub(probadualpos(ka,kb,va)) = Inf;
end;
for vb = 1:kb
    lb(probbdualpos(ka,kb,vb)) = -Inf;
    ub(probbdualpos(ka,kb,vb)) = Inf;
end;
ub(extracnsdualpos(ka,kb)) = Inf;

%% now do the optimization

opts = optimoptions(@linprog,'MaxIter',mi,'Algorithm','dual-simplex');
[rawmech,negrev,ef,output,lambda] = linprog(obj,A,b,[],[],lb,ub,[],opts);
rev = -negrev;

%% reformat the mechanism

allocamat = zeros(ka,kb);
allocbmat = zeros(ka,kb);
paymat = zeros(ka,kb);

for va = 1:ka
    for vb = 1:kb
        allocamat(va,vb) = rawmech(allocapos(ka,kb,va,vb));
        allocbmat(va,vb) = rawmech(allocbpos(ka,kb,va,vb));
        paymat(va,vb) = rawmech(paypos(ka,kb,va,vb));
    end;
end;

mech.allocamat = allocamat;
mech.allocbmat = allocbmat;
mech.paymat = paymat;

wcdist = zeros(ka,kb);
%% for va = 1:ka
 %%   for vb = 1:kb
 %%       wcdist(va,vb) = lambda.ineqlin(dualcnspos(va,vb));
 %%   end;
%% end;

%% revenue from selling the goods separately?  do it by brute force

maxreva = 0;
for va = 1:ka
    thisprice = valuesa(1,va);
    probbuy = 0;
    for vap = 1:ka
        if valuesa(1,vap) >= thisprice
            probbuy = probbuy + probsa(1,vap);
        end;
    end;
    thisreva = thisprice * probbuy;
    if thisreva > maxreva
        maxreva = thisreva;
    end;
end;

maxrevb = 0;
for vb = 1:kb
    thisprice = valuesb(1,vb);
    probbuy = 0;
    for vbp = 1:kb
        if valuesb(1,vbp) >= thisprice
            probbuy = probbuy + probsb(1,vbp);
        end;
    end;
    thisrevb = thisprice * probbuy;
    if thisrevb > maxrevb
        maxrevb = thisrevb;
    end;
end;

seprev = maxreva + maxrevb;










